import graphene

from .asset_key import GrapheneAssetKey
from .table import GrapheneTable, GrapheneTableSchema


class GrapheneMetadataItemDefinition(graphene.ObjectType):
    key = graphene.NonNull(graphene.String)
    value = graphene.NonNull(graphene.String)

    class Meta:
        name = "MetadataItemDefinition"


class GrapheneEventMetadataEntry(graphene.Interface):
    label = graphene.NonNull(graphene.String)
    description = graphene.String()

    class Meta:
        name = "EventMetadataEntry"


class GrapheneEventPathMetadataEntry(graphene.ObjectType):
    path = graphene.NonNull(graphene.String)

    class Meta:
        interfaces = (GrapheneEventMetadataEntry,)
        name = "EventPathMetadataEntry"


class GrapheneEventTableMetadataEntry(graphene.ObjectType):
    table = graphene.NonNull(GrapheneTable)

    class Meta:
        interfaces = (GrapheneEventMetadataEntry,)
        name = "EventTableMetadataEntry"


class GrapheneEventTableSchemaMetadataEntry(graphene.ObjectType):

    schema = graphene.NonNull(GrapheneTableSchema)

    class Meta:
        interfaces = (GrapheneEventMetadataEntry,)
        name = "EventTableSchemaMetadataEntry"


class GrapheneEventJsonMetadataEntry(graphene.ObjectType):
    jsonString = graphene.NonNull(graphene.String)

    class Meta:
        interfaces = (GrapheneEventMetadataEntry,)
        name = "EventJsonMetadataEntry"


class GrapheneEventTextMetadataEntry(graphene.ObjectType):
    text = graphene.NonNull(graphene.String)

    class Meta:
        interfaces = (GrapheneEventMetadataEntry,)
        name = "EventTextMetadataEntry"


class GrapheneEventUrlMetadataEntry(graphene.ObjectType):
    url = graphene.NonNull(graphene.String)

    class Meta:
        interfaces = (GrapheneEventMetadataEntry,)
        name = "EventUrlMetadataEntry"


class GrapheneEventMarkdownMetadataEntry(graphene.ObjectType):
    md_str = graphene.NonNull(graphene.String)

    class Meta:
        interfaces = (GrapheneEventMetadataEntry,)
        name = "EventMarkdownMetadataEntry"


class GrapheneEventPythonArtifactMetadataEntry(graphene.ObjectType):
    module = graphene.NonNull(graphene.String)
    name = graphene.NonNull(graphene.String)

    class Meta:
        interfaces = (GrapheneEventMetadataEntry,)
        name = "EventPythonArtifactMetadataEntry"


class GrapheneEventFloatMetadataEntry(graphene.ObjectType):
    floatValue = graphene.Field(graphene.Float)

    class Meta:
        interfaces = (GrapheneEventMetadataEntry,)
        name = "EventFloatMetadataEntry"


class GrapheneEventIntMetadataEntry(graphene.ObjectType):
    intValue = graphene.Field(
        graphene.Int, description="Nullable to allow graceful degrade on > 32 bit numbers"
    )
    intRepr = graphene.NonNull(
        graphene.String,
        description="String representation of the int to support greater than 32 bit",
    )

    class Meta:
        interfaces = (GrapheneEventMetadataEntry,)
        name = "EventIntMetadataEntry"


class GrapheneEventPipelineRunMetadataEntry(graphene.ObjectType):
    runId = graphene.NonNull(graphene.String)

    class Meta:
        interfaces = (GrapheneEventMetadataEntry,)
        name = "EventPipelineRunMetadataEntry"


class GrapheneEventAssetMetadataEntry(graphene.ObjectType):
    assetKey = graphene.NonNull(GrapheneAssetKey)

    class Meta:
        interfaces = (GrapheneEventMetadataEntry,)
        name = "EventAssetMetadataEntry"


def types():
    return [
        GrapheneEventMetadataEntry,
        GrapheneEventTableSchemaMetadataEntry,
        GrapheneEventTableMetadataEntry,
        GrapheneEventFloatMetadataEntry,
        GrapheneEventIntMetadataEntry,
        GrapheneEventJsonMetadataEntry,
        GrapheneEventMarkdownMetadataEntry,
        GrapheneMetadataItemDefinition,
        GrapheneEventPathMetadataEntry,
        GrapheneEventPythonArtifactMetadataEntry,
        GrapheneEventTextMetadataEntry,
        GrapheneEventUrlMetadataEntry,
        GrapheneEventPipelineRunMetadataEntry,
        GrapheneEventAssetMetadataEntry,
    ]



import copy

import yaml
from dagster import (
    AssetMaterialization,
    Output,
    execute_pipeline,
    job,
    lambda_solid,
    op,
    pipeline,
    repository,
)
from dagster.core.definitions.pipeline_base import InMemoryPipeline
from dagster.core.execution.api import execute_run
from dagster.core.storage.pipeline_run import PipelineRunStatus
from dagster.core.storage.tags import PARENT_RUN_ID_TAG, ROOT_RUN_ID_TAG
from dagster.core.test_utils import instance_for_test
from dagster.utils import Counter, traced_counter
from dagster_graphql.test.utils import (
    define_out_of_process_context,
    execute_dagster_graphql,
    infer_pipeline_selector,
    infer_repository_selector,
)
from dagster_graphql_tests.graphql.graphql_context_test_suite import (
    ExecutingGraphQLContextTestMatrix,
)

RUNS_QUERY = """
query PipelineRunsRootQuery($selector: PipelineSelector!) {
  pipelineOrError(params: $selector) {
    ... on Pipeline {
      name
      pipelineSnapshotId
      runs {
        ...RunHistoryRunFragment
      }
    }
  }
}

fragment RunHistoryRunFragment on PipelineRun {
  runId
  status
  repositoryOrigin {
    repositoryLocationName
    repositoryName
    repositoryLocationMetadata {
      key
      value
    }
  }
  pipeline {
    ...on PipelineReference { name }
  }
  executionPlan {
    steps {
      key
    }
  }
  runConfig
  runConfigYaml
  mode
  canTerminate
  tags {
    key
    value
  }
}
"""

DELETE_RUN_MUTATION = """
mutation DeleteRun($runId: String!) {
  deletePipelineRun(runId: $runId) {
    __typename
    ... on DeletePipelineRunSuccess {
      runId
    }
    ... on PythonError {
      message
    }
  }
}
"""

ALL_TAGS_QUERY = """
{
  pipelineRunTags {
    ... on PipelineTagAndValues {
      key
      values
    }
  }
}
"""

ALL_RUNS_QUERY = """
{
  pipelineRunsOrError {
    ... on PipelineRuns {
      results {
        runId
        pipelineSnapshotId
        pipeline {
          __typename
          ... on PipelineReference {
            name
            solidSelection
          }
        }
      }
    }
  }
}
"""


FILTERED_RUN_QUERY = """
query PipelineRunsRootQuery($filter: RunsFilter!) {
  pipelineRunsOrError(filter: $filter) {
    ... on PipelineRuns {
      results {
        runId
      }
    }
  }
}
"""

FILTERED_RUN_COUNT_QUERY = """
query PipelineRunsRootQuery($filter: RunsFilter!) {
  pipelineRunsOrError(filter: $filter) {
    ... on PipelineRuns {
      count
    }
  }
}
"""


RUN_GROUP_QUERY = """
query RunGroupQuery($runId: ID!) {
  runGroupOrError(runId: $runId) {
    ... on RunGroup {
      __typename
      rootRunId
      runs {
        runId
        pipeline {
          __typename
          ... on PipelineReference {
            name
          }
        }
      }
    }
    ... on RunGroupNotFoundError {
      __typename
      runId
      message
    }
  }
}
"""


ALL_RUN_GROUPS_QUERY = """
{
  runGroupsOrError {
    results {
      rootRunId
      runs {
        runId
        pipelineSnapshotId
        pipeline {
          __typename
          ... on PipelineReference {
            name
            solidSelection
          }
        }
      }
    }
  }
}
"""

REPOSITORY_RUNS_QUERY = """
query RepositoryRunsQuery($repositorySelector: RepositorySelector!) {
    repositoryOrError(repositorySelector: $repositorySelector) {
        ... on Repository {
            id
            name
            pipelines {
                id
                name
                runs(limit: 10) {
                    id
                    runId
                }
            }
        }
    }
}
"""

ASSET_RUNS_QUERY = """
query AssetRunsQuery($assetKey: AssetKeyInput!) {
    assetOrError(assetKey: $assetKey) {
        ... on Asset {
            assetMaterializations {
                label
                runOrError {
                    ... on PipelineRun {
                        id
                        runId
                    }
                }
            }
        }
    }
}
"""


def _get_runs_data(result, run_id):
    for run_data in result.data["pipelineOrError"]["runs"]:
        if run_data["runId"] == run_id:
            # so caller can delete keys
            return copy.deepcopy(run_data)

    return None


class TestGetRuns(ExecutingGraphQLContextTestMatrix):
    def test_get_runs_over_graphql(self, graphql_context):
        # This include needs to be here because its inclusion screws up
        # other code in this file which reads itself to load a repo
        from .utils import sync_execute_get_run_log_data

        selector = infer_pipeline_selector(graphql_context, "multi_mode_with_resources")

        payload_one = sync_execute_get_run_log_data(
            context=graphql_context,
            variables={
                "executionParams": {
                    "selector": selector,
                    "mode": "add_mode",
                    "runConfigData": {"resources": {"op": {"config": 2}}},
                    "executionMetadata": {"tags": [{"key": "fruit", "value": "apple"}]},
                }
            },
        )
        run_id_one = payload_one["run"]["runId"]

        read_context = graphql_context

        result = execute_dagster_graphql(read_context, RUNS_QUERY, variables={"selector": selector})

        runs = result.data["pipelineOrError"]["runs"]
        assert len(runs) == 1

        tags = runs[0]["tags"]
        assert len(tags) == 1

        assert tags[0]["key"] == "fruit"
        assert tags[0]["value"] == "apple"

        origin = runs[0]["repositoryOrigin"]
        assert origin
        assert origin["repositoryLocationName"] == selector["repositoryLocationName"]
        assert origin["repositoryName"] == selector["repositoryName"]
        assert origin["repositoryLocationMetadata"]

        payload_two = sync_execute_get_run_log_data(
            context=graphql_context,
            variables={
                "executionParams": {
                    "selector": selector,
                    "mode": "add_mode",
                    "runConfigData": {"resources": {"op": {"config": 3}}},
                    "executionMetadata": {"tags": [{"key": "veggie", "value": "carrot"}]},
                }
            },
        )

        run_id_two = payload_two["run"]["runId"]

        result = execute_dagster_graphql(read_context, RUNS_QUERY, variables={"selector": selector})

        runs = result.data["pipelineOrError"]["runs"]
        assert len(runs) == 2

        all_tags_result = execute_dagster_graphql(read_context, ALL_TAGS_QUERY)
        tags = all_tags_result.data["pipelineRunTags"]

        assert len(tags) == 2
        tags_dict = {item["key"]: item["values"] for item in tags}

        assert tags_dict == {
            "fruit": ["apple"],
            "veggie": ["carrot"],
        }

        # delete the second run
        result = execute_dagster_graphql(
            read_context, DELETE_RUN_MUTATION, variables={"runId": run_id_two}
        )
        assert result.data["deletePipelineRun"]["__typename"] == "DeletePipelineRunSuccess"
        assert result.data["deletePipelineRun"]["runId"] == run_id_two

        # query it back out
        result = execute_dagster_graphql(read_context, RUNS_QUERY, variables={"selector": selector})

        # first is the same
        run_one_data = _get_runs_data(result, run_id_one)
        assert run_one_data

        # second is gone
        run_two_data = _get_runs_data(result, run_id_two)
        assert run_two_data is None

        # try to delete the second run again
        execute_dagster_graphql(read_context, DELETE_RUN_MUTATION, variables={"runId": run_id_two})

        result = execute_dagster_graphql(
            read_context, DELETE_RUN_MUTATION, variables={"runId": run_id_two}
        )
        assert result.data["deletePipelineRun"]["__typename"] == "RunNotFoundError"

    def test_run_config(self, graphql_context):
        # This include needs to be here because its inclusion screws up
        # other code in this file which reads itself to load a repo
        from .utils import sync_execute_get_run_log_data

        selector = infer_pipeline_selector(graphql_context, "multi_mode_with_resources")

        sync_execute_get_run_log_data(
            context=graphql_context,
            variables={
                "executionParams": {
                    "selector": selector,
                    "mode": "add_mode",
                    "runConfigData": {"resources": {"op": {"config": 2}}},
                }
            },
        )
        result = execute_dagster_graphql(
            graphql_context, RUNS_QUERY, variables={"selector": selector}
        )

        runs = result.data["pipelineOrError"]["runs"]
        assert len(runs) == 1
        run = runs[0]
        assert run["runConfig"] == {"resources": {"op": {"config": 2}}}
        assert run["runConfigYaml"] == yaml.safe_dump(
            run["runConfig"], default_flow_style=False, allow_unicode=True
        )


def get_repo_at_time_1():
    @lambda_solid
    def solid_A():
        pass

    @lambda_solid
    def solid_B():
        pass

    @pipeline
    def evolving_pipeline():
        solid_A()
        solid_B()

    @pipeline
    def foo_pipeline():
        solid_A()

    @repository
    def evolving_repo():
        return [evolving_pipeline, foo_pipeline]

    return evolving_repo


def get_repo_at_time_2():
    @lambda_solid
    def solid_A():
        pass

    @lambda_solid
    def solid_B_prime():
        pass

    @pipeline
    def evolving_pipeline():
        solid_A()
        solid_B_prime()

    @pipeline
    def bar_pipeline():
        solid_A()

    @repository
    def evolving_repo():
        return [evolving_pipeline, bar_pipeline]

    return evolving_repo


def get_asset_repo():
    @op
    def foo():
        yield AssetMaterialization(asset_key="foo", description="foo")
        yield Output(None)

    @job
    def foo_job():
        foo()

    @repository
    def asset_repo():
        return [foo_job]

    return asset_repo


def test_runs_over_time():
    with instance_for_test() as instance:

        repo_1 = get_repo_at_time_1()

        full_evolve_run_id = execute_pipeline(
            repo_1.get_pipeline("evolving_pipeline"), instance=instance
        ).run_id
        foo_run_id = execute_pipeline(repo_1.get_pipeline("foo_pipeline"), instance=instance).run_id
        evolve_a_run_id = execute_pipeline(
            repo_1.get_pipeline("evolving_pipeline").get_pipeline_subset_def({"solid_A"}),
            instance=instance,
        ).run_id
        evolve_b_run_id = execute_pipeline(
            repo_1.get_pipeline("evolving_pipeline").get_pipeline_subset_def({"solid_B"}),
            instance=instance,
        ).run_id

        with define_out_of_process_context(
            __file__, "get_repo_at_time_1", instance
        ) as context_at_time_1:
            result = execute_dagster_graphql(context_at_time_1, ALL_RUNS_QUERY)
            assert result.data

            t1_runs = {run["runId"]: run for run in result.data["pipelineRunsOrError"]["results"]}

            assert t1_runs[full_evolve_run_id]["pipeline"] == {
                "__typename": "PipelineSnapshot",
                "name": "evolving_pipeline",
                "solidSelection": None,
            }

            assert t1_runs[foo_run_id]["pipeline"] == {
                "__typename": "PipelineSnapshot",
                "name": "foo_pipeline",
                "solidSelection": None,
            }

            assert t1_runs[evolve_a_run_id]["pipeline"] == {
                "__typename": "PipelineSnapshot",
                "name": "evolving_pipeline",
                "solidSelection": ["solid_A"],
            }

            assert t1_runs[evolve_b_run_id]["pipeline"] == {
                "__typename": "PipelineSnapshot",
                "name": "evolving_pipeline",
                "solidSelection": ["solid_B"],
            }

        with define_out_of_process_context(
            __file__, "get_repo_at_time_2", instance
        ) as context_at_time_2:
            result = execute_dagster_graphql(context_at_time_2, ALL_RUNS_QUERY)
            assert result.data

            t2_runs = {run["runId"]: run for run in result.data["pipelineRunsOrError"]["results"]}

            assert t2_runs[full_evolve_run_id]["pipeline"] == {
                "__typename": "PipelineSnapshot",
                "name": "evolving_pipeline",
                "solidSelection": None,
            }

            assert t2_runs[evolve_a_run_id]["pipeline"] == {
                "__typename": "PipelineSnapshot",
                "name": "evolving_pipeline",
                "solidSelection": ["solid_A"],
            }
            # pipeline name changed
            assert t2_runs[foo_run_id]["pipeline"] == {
                "__typename": "PipelineSnapshot",
                "name": "foo_pipeline",
                "solidSelection": None,
            }
            # subset no longer valid - b renamed
            assert t2_runs[evolve_b_run_id]["pipeline"] == {
                "__typename": "PipelineSnapshot",
                "name": "evolving_pipeline",
                "solidSelection": ["solid_B"],
            }


def test_run_groups_over_time():
    with instance_for_test() as instance:
        repo_1 = get_repo_at_time_1()

        full_evolve_run_id = execute_pipeline(
            repo_1.get_pipeline("evolving_pipeline"), instance=instance
        ).run_id
        foo_run_id = execute_pipeline(repo_1.get_pipeline("foo_pipeline"), instance=instance).run_id
        evolve_a_run_id = execute_pipeline(
            repo_1.get_pipeline("evolving_pipeline").get_pipeline_subset_def({"solid_A"}),
            instance=instance,
        ).run_id
        evolve_b_run_id = execute_pipeline(
            repo_1.get_pipeline("evolving_pipeline").get_pipeline_subset_def({"solid_B"}),
            instance=instance,
        ).run_id

        with define_out_of_process_context(
            __file__, "get_repo_at_time_1", instance
        ) as context_at_time_1:

            result = execute_dagster_graphql(context_at_time_1, ALL_RUN_GROUPS_QUERY)
            assert result.data
            assert "runGroupsOrError" in result.data
            assert "results" in result.data["runGroupsOrError"]
            assert len(result.data["runGroupsOrError"]["results"]) == 4

            t1_runs = {
                run["runId"]: run
                for group in result.data["runGroupsOrError"]["results"]
                for run in group["runs"]
            }

            # test full_evolve_run_id
            assert t1_runs[full_evolve_run_id]["pipeline"] == {
                "__typename": "PipelineSnapshot",
                "name": "evolving_pipeline",
                "solidSelection": None,
            }

            # test foo_run_id
            assert t1_runs[foo_run_id]["pipeline"] == {
                "__typename": "PipelineSnapshot",
                "name": "foo_pipeline",
                "solidSelection": None,
            }

            # test evolve_a_run_id
            assert t1_runs[evolve_a_run_id]["pipeline"] == {
                "__typename": "PipelineSnapshot",
                "name": "evolving_pipeline",
                "solidSelection": ["solid_A"],
            }
            assert t1_runs[evolve_a_run_id]["pipelineSnapshotId"]

            # test evolve_b_run_id
            assert t1_runs[evolve_b_run_id]["pipeline"] == {
                "__typename": "PipelineSnapshot",
                "name": "evolving_pipeline",
                "solidSelection": ["solid_B"],
            }

        with define_out_of_process_context(
            __file__, "get_repo_at_time_2", instance
        ) as context_at_time_2:
            result = execute_dagster_graphql(context_at_time_2, ALL_RUN_GROUPS_QUERY)
            assert "runGroupsOrError" in result.data
            assert "results" in result.data["runGroupsOrError"]
            assert len(result.data["runGroupsOrError"]["results"]) == 4

            t2_runs = {
                run["runId"]: run
                for group in result.data["runGroupsOrError"]["results"]
                for run in group["runs"]
            }

            # test full_evolve_run_id
            assert t2_runs[full_evolve_run_id]["pipeline"] == {
                "__typename": "PipelineSnapshot",
                "name": "evolving_pipeline",
                "solidSelection": None,
            }

            # test evolve_a_run_id
            assert t2_runs[evolve_a_run_id]["pipeline"] == {
                "__typename": "PipelineSnapshot",
                "name": "evolving_pipeline",
                "solidSelection": ["solid_A"],
            }
            assert t2_runs[evolve_a_run_id]["pipelineSnapshotId"]

            # names same
            assert (
                t1_runs[full_evolve_run_id]["pipeline"]["name"]
                == t2_runs[evolve_a_run_id]["pipeline"]["name"]
            )

            # snapshots differ
            assert (
                t1_runs[full_evolve_run_id]["pipelineSnapshotId"]
                != t2_runs[evolve_a_run_id]["pipelineSnapshotId"]
            )

            # pipeline name changed
            assert t2_runs[foo_run_id]["pipeline"] == {
                "__typename": "PipelineSnapshot",
                "name": "foo_pipeline",
                "solidSelection": None,
            }
            # subset no longer valid - b renamed
            assert t2_runs[evolve_b_run_id]["pipeline"] == {
                "__typename": "PipelineSnapshot",
                "name": "evolving_pipeline",
                "solidSelection": ["solid_B"],
            }


def test_filtered_runs():
    with instance_for_test() as instance:
        repo = get_repo_at_time_1()
        run_id_1 = execute_pipeline(
            repo.get_pipeline("foo_pipeline"), instance=instance, tags={"run": "one"}
        ).run_id
        run_id_2 = execute_pipeline(
            repo.get_pipeline("foo_pipeline"), instance=instance, tags={"run": "two"}
        ).run_id
        with define_out_of_process_context(__file__, "get_repo_at_time_1", instance) as context:
            result = execute_dagster_graphql(
                context, FILTERED_RUN_QUERY, variables={"filter": {"runIds": [run_id_1]}}
            )
            assert result.data
            run_ids = [run["runId"] for run in result.data["pipelineRunsOrError"]["results"]]
            assert len(run_ids) == 1
            assert run_ids[0] == run_id_1

            result = execute_dagster_graphql(
                context,
                FILTERED_RUN_QUERY,
                variables={"filter": {"tags": [{"key": "run", "value": "one"}]}},
            )
            assert result.data
            run_ids = [run["runId"] for run in result.data["pipelineRunsOrError"]["results"]]
            assert len(run_ids) == 1
            assert run_ids[0] == run_id_1

            # test multiple run ids
            result = execute_dagster_graphql(
                context, FILTERED_RUN_QUERY, variables={"filter": {"runIds": [run_id_1, run_id_2]}}
            )
            assert result.data
            run_ids = [run["runId"] for run in result.data["pipelineRunsOrError"]["results"]]
            assert len(run_ids) == 2
            assert set(run_ids) == set([run_id_1, run_id_2])


def test_filtered_runs_status():
    with instance_for_test() as instance:
        repo = get_repo_at_time_1()
        _ = instance.create_run_for_pipeline(
            repo.get_pipeline("foo_pipeline"), status=PipelineRunStatus.STARTED
        ).run_id
        run_id_2 = instance.create_run_for_pipeline(
            repo.get_pipeline("foo_pipeline"), status=PipelineRunStatus.FAILURE
        ).run_id
        with define_out_of_process_context(__file__, "get_repo_at_time_1", instance) as context:
            result = execute_dagster_graphql(
                context, FILTERED_RUN_QUERY, variables={"filter": {"statuses": ["FAILURE"]}}
            )
            assert result.data
            run_ids = [run["runId"] for run in result.data["pipelineRunsOrError"]["results"]]
            assert len(run_ids) == 1
            assert run_ids[0] == run_id_2


def test_filtered_runs_multiple_statuses():
    with instance_for_test() as instance:
        repo = get_repo_at_time_1()
        _ = instance.create_run_for_pipeline(
            repo.get_pipeline("foo_pipeline"), status=PipelineRunStatus.STARTED
        ).run_id
        run_id_2 = instance.create_run_for_pipeline(
            repo.get_pipeline("foo_pipeline"), status=PipelineRunStatus.FAILURE
        ).run_id
        run_id_3 = instance.create_run_for_pipeline(
            repo.get_pipeline("foo_pipeline"), status=PipelineRunStatus.SUCCESS
        ).run_id
        with define_out_of_process_context(__file__, "get_repo_at_time_1", instance) as context:
            result = execute_dagster_graphql(
                context,
                FILTERED_RUN_QUERY,
                variables={"filter": {"statuses": ["FAILURE", "SUCCESS"]}},
            )
            assert result.data
            run_ids = [run["runId"] for run in result.data["pipelineRunsOrError"]["results"]]
            assert len(run_ids) == 2
            assert run_id_2 in run_ids
            assert run_id_3 in run_ids


def test_filtered_runs_multiple_filters():
    with instance_for_test() as instance:
        repo = get_repo_at_time_1()

        started_run_with_tags = instance.create_run_for_pipeline(
            repo.get_pipeline("foo_pipeline"), status=PipelineRunStatus.STARTED, tags={"foo": "bar"}
        )
        failed_run_with_tags = instance.create_run_for_pipeline(
            repo.get_pipeline("foo_pipeline"), status=PipelineRunStatus.FAILURE, tags={"foo": "bar"}
        )
        started_run_without_tags = instance.create_run_for_pipeline(
            repo.get_pipeline("foo_pipeline"),
            status=PipelineRunStatus.STARTED,
            tags={"baz": "boom"},
        )

        with define_out_of_process_context(__file__, "get_repo_at_time_1", instance) as context:
            result = execute_dagster_graphql(
                context,
                FILTERED_RUN_QUERY,
                variables={
                    "filter": {"statuses": ["STARTED"], "tags": [{"key": "foo", "value": "bar"}]}
                },
            )
            assert result.data
            run_ids = [run["runId"] for run in result.data["pipelineRunsOrError"]["results"]]
            assert len(run_ids) == 1
            assert started_run_with_tags.run_id in run_ids
            assert failed_run_with_tags.run_id not in run_ids
            assert started_run_without_tags.run_id not in run_ids


def test_filtered_runs_count():
    with instance_for_test() as instance:
        repo = get_repo_at_time_1()
        instance.create_run_for_pipeline(  # pylint: disable=expression-not-assigned
            repo.get_pipeline("foo_pipeline"), status=PipelineRunStatus.STARTED
        ).run_id
        instance.create_run_for_pipeline(  # pylint: disable=expression-not-assigned
            repo.get_pipeline("foo_pipeline"), status=PipelineRunStatus.FAILURE
        ).run_id
        with define_out_of_process_context(__file__, "get_repo_at_time_1", instance) as context:
            result = execute_dagster_graphql(
                context, FILTERED_RUN_COUNT_QUERY, variables={"filter": {"statuses": ["FAILURE"]}}
            )
            assert result.data
            count = result.data["pipelineRunsOrError"]["count"]
            assert count == 1


def test_run_group():
    with instance_for_test() as instance:
        repo = get_repo_at_time_1()
        foo_pipeline = repo.get_pipeline("foo_pipeline")
        runs = [execute_pipeline(foo_pipeline, instance=instance)]
        root_run_id = runs[-1].run_id
        for _ in range(3):
            # https://github.com/dagster-io/dagster/issues/2433
            run = instance.create_run_for_pipeline(
                foo_pipeline,
                parent_run_id=root_run_id,
                root_run_id=root_run_id,
                tags={PARENT_RUN_ID_TAG: root_run_id, ROOT_RUN_ID_TAG: root_run_id},
            )
            execute_run(InMemoryPipeline(foo_pipeline), run, instance)
            runs.append(run)

        with define_out_of_process_context(
            __file__, "get_repo_at_time_1", instance
        ) as context_at_time_1:
            result_one = execute_dagster_graphql(
                context_at_time_1,
                RUN_GROUP_QUERY,
                variables={"runId": root_run_id},
            )
            assert result_one.data["runGroupOrError"]["__typename"] == "RunGroup"

            assert len(result_one.data["runGroupOrError"]["runs"]) == 4

            result_two = execute_dagster_graphql(
                context_at_time_1,
                RUN_GROUP_QUERY,
                variables={"runId": runs[-1].run_id},
            )
            assert result_one.data["runGroupOrError"]["__typename"] == "RunGroup"
            assert len(result_two.data["runGroupOrError"]["runs"]) == 4

            assert (
                result_one.data["runGroupOrError"]["rootRunId"]
                == result_two.data["runGroupOrError"]["rootRunId"]
            )
            assert (
                result_one.data["runGroupOrError"]["runs"]
                == result_two.data["runGroupOrError"]["runs"]
            )


def test_run_group_not_found():
    with instance_for_test() as instance:
        with define_out_of_process_context(
            __file__, "get_repo_at_time_1", instance
        ) as context_at_time_1:

            result = execute_dagster_graphql(
                context_at_time_1,
                RUN_GROUP_QUERY,
                variables={"runId": "foo"},
            )
            assert result.data
            assert result.data["runGroupOrError"]
            assert result.data["runGroupOrError"]["__typename"] == "RunGroupNotFoundError"
            assert result.data["runGroupOrError"]["runId"] == "foo"
            assert result.data["runGroupOrError"][
                "message"
            ] == "Run group of run {run_id} could not be found.".format(run_id="foo")


def test_run_groups():
    with instance_for_test() as instance:
        repo = get_repo_at_time_1()
        foo_pipeline = repo.get_pipeline("foo_pipeline")

        root_run_ids = [execute_pipeline(foo_pipeline, instance=instance).run_id for i in range(3)]

        for _ in range(5):
            for root_run_id in root_run_ids:
                execute_pipeline(
                    foo_pipeline,
                    tags={PARENT_RUN_ID_TAG: root_run_id, ROOT_RUN_ID_TAG: root_run_id},
                    instance=instance,
                )

        with define_out_of_process_context(
            __file__, "get_repo_at_time_1", instance
        ) as context_at_time_1:
            result = execute_dagster_graphql(context_at_time_1, ALL_RUN_GROUPS_QUERY)

            assert result.data
            assert "runGroupsOrError" in result.data
            assert "results" in result.data["runGroupsOrError"]
            assert len(result.data["runGroupsOrError"]["results"]) == 3
            for run_group in result.data["runGroupsOrError"]["results"]:
                assert run_group["rootRunId"] in root_run_ids
                assert len(run_group["runs"]) == 6


def test_repository_batching():
    with instance_for_test() as instance:
        repo = get_repo_at_time_1()
        foo_pipeline = repo.get_pipeline("foo_pipeline")
        evolving_pipeline = repo.get_pipeline("evolving_pipeline")
        foo_run_ids = [execute_pipeline(foo_pipeline, instance=instance).run_id for i in range(3)]
        evolving_run_ids = [
            execute_pipeline(evolving_pipeline, instance=instance).run_id for i in range(2)
        ]
        with define_out_of_process_context(__file__, "get_repo_at_time_1", instance) as context:
            traced_counter.set(Counter())
            result = execute_dagster_graphql(
                context,
                REPOSITORY_RUNS_QUERY,
                variables={"repositorySelector": infer_repository_selector(context)},
            )
            assert result.data
            assert "repositoryOrError" in result.data
            assert "pipelines" in result.data["repositoryOrError"]
            pipelines = result.data["repositoryOrError"]["pipelines"]
            assert len(pipelines) == 2
            pipeline_runs = {pipeline["name"]: pipeline["runs"] for pipeline in pipelines}
            assert len(pipeline_runs["foo_pipeline"]) == 3
            assert len(pipeline_runs["evolving_pipeline"]) == 2
            assert set(foo_run_ids) == set(run["runId"] for run in pipeline_runs["foo_pipeline"])
            assert set(evolving_run_ids) == set(
                run["runId"] for run in pipeline_runs["evolving_pipeline"]
            )
            counter = traced_counter.get()
            counts = counter.counts()
            assert counts
            assert len(counts) == 1
            # We should have a single batch call to fetch run records, instead of 3 separate calls
            # to fetch run records (which is fetched to instantiate GrapheneRun)
            assert counts.get("DagsterInstance.get_run_records") == 1


def test_asset_batching():
    with instance_for_test() as instance:
        repo = get_asset_repo()
        foo_job = repo.get_job("foo_job")
        for _ in range(3):
            foo_job.execute_in_process(instance=instance)
        with define_out_of_process_context(__file__, "asset_repo", instance) as context:
            traced_counter.set(Counter())
            result = execute_dagster_graphql(
                context, ASSET_RUNS_QUERY, variables={"assetKey": {"path": ["foo"]}}
            )
            assert result.data
            assert "assetOrError" in result.data
            assert "assetMaterializations" in result.data["assetOrError"]
            materializations = result.data["assetOrError"]["assetMaterializations"]
            assert len(materializations) == 3
            counter = traced_counter.get()
            counts = counter.counts()
            assert counts
            assert counts.get("DagsterInstance.get_run_records") == 1



# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import Snapshot

snapshots = Snapshot()

snapshots['TestAssetAwareEventLog.test_all_asset_keys[asset_aware_instance_in_process_env] 1'] = {
    'assetsOrError': {
        '__typename': 'AssetConnection',
        'nodes': [
            {
                'key': {
                    'path': [
                        'a'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'asset_1'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'asset_2'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'asset_3'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'asset_one'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'asset_two'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'asset_yields_observation'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'b'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'c'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'downstream_static_partitioned_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'downstream_time_partitioned_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'dummy_source_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'first_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'hanging_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'never_runs_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'upstream_static_partitioned_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'upstream_time_partitioned_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'yield_partition_materialization'
                    ]
                }
            }
        ]
    }
}

snapshots['TestAssetAwareEventLog.test_all_asset_keys[postgres_with_default_run_launcher_managed_grpc_env] 1'] = {
    'assetsOrError': {
        '__typename': 'AssetConnection',
        'nodes': [
            {
                'key': {
                    'path': [
                        'a'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'asset_1'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'asset_2'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'asset_3'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'asset_one'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'asset_two'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'asset_yields_observation'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'b'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'c'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'downstream_static_partitioned_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'downstream_time_partitioned_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'dummy_source_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'first_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'hanging_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'never_runs_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'upstream_static_partitioned_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'upstream_time_partitioned_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'yield_partition_materialization'
                    ]
                }
            }
        ]
    }
}

snapshots['TestAssetAwareEventLog.test_all_asset_keys[sqlite_with_default_run_launcher_managed_grpc_env] 1'] = {
    'assetsOrError': {
        '__typename': 'AssetConnection',
        'nodes': [
            {
                'key': {
                    'path': [
                        'a'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'asset_1'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'asset_2'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'asset_3'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'asset_one'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'asset_two'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'asset_yields_observation'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'b'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'c'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'downstream_static_partitioned_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'downstream_time_partitioned_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'dummy_source_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'first_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'hanging_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'never_runs_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'upstream_static_partitioned_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'upstream_time_partitioned_asset'
                    ]
                }
            },
            {
                'key': {
                    'path': [
                        'yield_partition_materialization'
                    ]
                }
            }
        ]
    }
}



# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import Snapshot

snapshots = Snapshot()

snapshots['TestComposites.test_composites[non_launchable_in_memory_instance_lazy_repository] 1'] = {
    'pipelineOrError': {
        '__typename': 'Pipeline',
        'name': 'composites_pipeline',
        'solidHandles': [
            {
                'handleID': 'add_four',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'add_four',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'div_four'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1.adder_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1.adder_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2.adder_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2.adder_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'div_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'div_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'div_1'
                            },
                            {
                                'name': 'div_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'add_four'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'div_four',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four.div_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'div_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'div_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four.div_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'div_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'div_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            }
        ]
    }
}

snapshots['TestComposites.test_composites[non_launchable_in_memory_instance_managed_grpc_env] 1'] = {
    'pipelineOrError': {
        '__typename': 'Pipeline',
        'name': 'composites_pipeline',
        'solidHandles': [
            {
                'handleID': 'add_four',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'add_four',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'div_four'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1.adder_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1.adder_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2.adder_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2.adder_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'div_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'div_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'div_1'
                            },
                            {
                                'name': 'div_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'add_four'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'div_four',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four.div_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'div_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'div_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four.div_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'div_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'div_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            }
        ]
    }
}

snapshots['TestComposites.test_composites[non_launchable_in_memory_instance_multi_location] 1'] = {
    'pipelineOrError': {
        '__typename': 'Pipeline',
        'name': 'composites_pipeline',
        'solidHandles': [
            {
                "handleID": "add_four",
                "solid": {
                    "definition": {
                        "inputMappings": [
                            {
                                "definition": {"name": "num"},
                                "mappedInput": {
                                    "definition": {"name": "num"},
                                    "solid": {"name": "adder_1"},
                                },
                            }
                        ],
                        "outputMappings": [
                            {
                                "definition": {"name": "result"},
                                "mappedOutput": {
                                    "definition": {"name": "result"},
                                    "solid": {"name": "adder_2"},
                                },
                            }
                        ],
                        "solids": [{"name": "adder_1"}, {"name": "adder_2"}],
                    },
                    "inputs": [{"definition": {"name": "num"}, "dependsOn": []}],
                    "name": "add_four",
                    "outputs": [
                        {
                            "definition": {"name": "result"},
                            "dependedBy": [{"solid": {"name": "div_four"}}],
                        }
                    ],
                },
            },
            {
                "handleID": "add_four.adder_1",
                "solid": {
                    "definition": {
                        "inputMappings": [
                            {
                                "definition": {"name": "num"},
                                "mappedInput": {
                                    "definition": {"name": "num"},
                                    "solid": {"name": "adder_1"},
                                },
                            }
                        ],
                        "outputMappings": [
                            {
                                "definition": {"name": "result"},
                                "mappedOutput": {
                                    "definition": {"name": "result"},
                                    "solid": {"name": "adder_2"},
                                },
                            }
                        ],
                        "solids": [{"name": "adder_1"}, {"name": "adder_2"}],
                    },
                    "inputs": [{"definition": {"name": "num"}, "dependsOn": []}],
                    "name": "adder_1",
                    "outputs": [
                        {
                            "definition": {"name": "result"},
                            "dependedBy": [{"solid": {"name": "adder_2"}}],
                        }
                    ],
                },
            },
            {
                "handleID": "add_four.adder_1.adder_1",
                "solid": {
                    "definition": {},
                    "inputs": [{"definition": {"name": "num"}, "dependsOn": []}],
                    "name": "adder_1",
                    "outputs": [
                        {
                            "definition": {"name": "result"},
                            "dependedBy": [{"solid": {"name": "adder_2"}}],
                        }
                    ],
                },
            },
            {
                "handleID": "add_four.adder_1.adder_2",
                "solid": {
                    "definition": {},
                    "inputs": [
                        {
                            "definition": {"name": "num"},
                            "dependsOn": [{"solid": {"name": "adder_1"}}],
                        }
                    ],
                    "name": "adder_2",
                    "outputs": [{"definition": {"name": "result"}, "dependedBy": []}],
                },
            },
            {
                "handleID": "add_four.adder_2",
                "solid": {
                    "definition": {
                        "inputMappings": [
                            {
                                "definition": {"name": "num"},
                                "mappedInput": {
                                    "definition": {"name": "num"},
                                    "solid": {"name": "adder_1"},
                                },
                            }
                        ],
                        "outputMappings": [
                            {
                                "definition": {"name": "result"},
                                "mappedOutput": {
                                    "definition": {"name": "result"},
                                    "solid": {"name": "adder_2"},
                                },
                            }
                        ],
                        "solids": [{"name": "adder_1"}, {"name": "adder_2"}],
                    },
                    "inputs": [
                        {
                            "definition": {"name": "num"},
                            "dependsOn": [{"solid": {"name": "adder_1"}}],
                        }
                    ],
                    "name": "adder_2",
                    "outputs": [{"definition": {"name": "result"}, "dependedBy": []}],
                },
            },
            {
                "handleID": "add_four.adder_2.adder_1",
                "solid": {
                    "definition": {},
                    "inputs": [{"definition": {"name": "num"}, "dependsOn": []}],
                    "name": "adder_1",
                    "outputs": [
                        {
                            "definition": {"name": "result"},
                            "dependedBy": [{"solid": {"name": "adder_2"}}],
                        }
                    ],
                },
            },
            {
                "handleID": "add_four.adder_2.adder_2",
                "solid": {
                    "definition": {},
                    "inputs": [
                        {
                            "definition": {"name": "num"},
                            "dependsOn": [{"solid": {"name": "adder_1"}}],
                        }
                    ],
                    "name": "adder_2",
                    "outputs": [{"definition": {"name": "result"}, "dependedBy": []}],
                },
            },
            {
                "handleID": "div_four",
                "solid": {
                    "definition": {
                        "inputMappings": [
                            {
                                "definition": {"name": "num"},
                                "mappedInput": {
                                    "definition": {"name": "num"},
                                    "solid": {"name": "div_1"},
                                },
                            }
                        ],
                        "outputMappings": [
                            {
                                "definition": {"name": "result"},
                                "mappedOutput": {
                                    "definition": {"name": "result"},
                                    "solid": {"name": "div_2"},
                                },
                            }
                        ],
                        "solids": [{"name": "div_1"}, {"name": "div_2"}],
                    },
                    "inputs": [
                        {
                            "definition": {"name": "num"},
                            "dependsOn": [{"solid": {"name": "add_four"}}],
                        }
                    ],
                    "name": "div_four",
                    "outputs": [{"definition": {"name": "result"}, "dependedBy": []}],
                },
            },
            {
                "handleID": "div_four.div_1",
                "solid": {
                    "definition": {},
                    "inputs": [{"definition": {"name": "num"}, "dependsOn": []}],
                    "name": "div_1",
                    "outputs": [
                        {
                            "definition": {"name": "result"},
                            "dependedBy": [{"solid": {"name": "div_2"}}],
                        }
                    ],
                },
            },
            {
                "handleID": "div_four.div_2",
                "solid": {
                    "definition": {},
                    "inputs": [
                        {"definition": {"name": "num"}, "dependsOn": [{"solid": {"name": "div_1"}}]}
                    ],
                    "name": "div_2",
                    "outputs": [{"definition": {"name": "result"}, "dependedBy": []}],
                },
            },
        ],
    }
}

snapshots["TestComposites.test_composites[non_launchable_sqlite_instance_managed_grpc_env] 1"] = {
    "pipelineOrError": {
        "__typename": "Pipeline",
        "name": "composites_pipeline",
        "solidHandles": [
            {
                "handleID": "add_four",
                "solid": {
                    "definition": {
                        "inputMappings": [
                            {
                                "definition": {"name": "num"},
                                "mappedInput": {
                                    "definition": {"name": "num"},
                                    "solid": {"name": "adder_1"},
                                },
                            }
                        ],
                        "outputMappings": [
                            {
                                "definition": {"name": "result"},
                                "mappedOutput": {
                                    "definition": {"name": "result"},
                                    "solid": {"name": "adder_2"},
                                },
                            }
                        ],
                        "solids": [{"name": "adder_1"}, {"name": "adder_2"}],
                    },
                    "inputs": [{"definition": {"name": "num"}, "dependsOn": []}],
                    "name": "add_four",
                    "outputs": [
                        {
                            "definition": {"name": "result"},
                            "dependedBy": [{"solid": {"name": "div_four"}}],
                        }
                    ],
                },
            },
            {
                "handleID": "add_four.adder_1",
                "solid": {
                    "definition": {
                        "inputMappings": [
                            {
                                "definition": {"name": "num"},
                                "mappedInput": {
                                    "definition": {"name": "num"},
                                    "solid": {"name": "adder_1"},
                                },
                            }
                        ],
                        "outputMappings": [
                            {
                                "definition": {"name": "result"},
                                "mappedOutput": {
                                    "definition": {"name": "result"},
                                    "solid": {"name": "adder_2"},
                                },
                            }
                        ],
                        "solids": [{"name": "adder_1"}, {"name": "adder_2"}],
                    },
                    "inputs": [{"definition": {"name": "num"}, "dependsOn": []}],
                    "name": "adder_1",
                    "outputs": [
                        {
                            "definition": {"name": "result"},
                            "dependedBy": [{"solid": {"name": "adder_2"}}],
                        }
                    ],
                },
            },
            {
                "handleID": "add_four.adder_1.adder_1",
                "solid": {
                    "definition": {},
                    "inputs": [{"definition": {"name": "num"}, "dependsOn": []}],
                    "name": "adder_1",
                    "outputs": [
                        {
                            "definition": {"name": "result"},
                            "dependedBy": [{"solid": {"name": "adder_2"}}],
                        }
                    ],
                },
            },
            {
                "handleID": "add_four.adder_1.adder_2",
                "solid": {
                    "definition": {},
                    "inputs": [
                        {
                            "definition": {"name": "num"},
                            "dependsOn": [{"solid": {"name": "adder_1"}}],
                        }
                    ],
                    "name": "adder_2",
                    "outputs": [{"definition": {"name": "result"}, "dependedBy": []}],
                },
            },
            {
                "handleID": "add_four.adder_2",
                "solid": {
                    "definition": {
                        "inputMappings": [
                            {
                                "definition": {"name": "num"},
                                "mappedInput": {
                                    "definition": {"name": "num"},
                                    "solid": {"name": "adder_1"},
                                },
                            }
                        ],
                        "outputMappings": [
                            {
                                "definition": {"name": "result"},
                                "mappedOutput": {
                                    "definition": {"name": "result"},
                                    "solid": {"name": "adder_2"},
                                },
                            }
                        ],
                        "solids": [{"name": "adder_1"}, {"name": "adder_2"}],
                    },
                    "inputs": [
                        {
                            "definition": {"name": "num"},
                            "dependsOn": [{"solid": {"name": "adder_1"}}],
                        }
                    ],
                    "name": "adder_2",
                    "outputs": [{"definition": {"name": "result"}, "dependedBy": []}],
                },
            },
            {
                "handleID": "add_four.adder_2.adder_1",
                "solid": {
                    "definition": {},
                    "inputs": [{"definition": {"name": "num"}, "dependsOn": []}],
                    "name": "adder_1",
                    "outputs": [
                        {
                            "definition": {"name": "result"},
                            "dependedBy": [{"solid": {"name": "adder_2"}}],
                        }
                    ],
                },
            },
            {
                "handleID": "add_four.adder_2.adder_2",
                "solid": {
                    "definition": {},
                    "inputs": [
                        {
                            "definition": {"name": "num"},
                            "dependsOn": [{"solid": {"name": "adder_1"}}],
                        }
                    ],
                    "name": "adder_2",
                    "outputs": [{"definition": {"name": "result"}, "dependedBy": []}],
                },
            },
            {
                "handleID": "div_four",
                "solid": {
                    "definition": {
                        "inputMappings": [
                            {
                                "definition": {"name": "num"},
                                "mappedInput": {
                                    "definition": {"name": "num"},
                                    "solid": {"name": "div_1"},
                                },
                            }
                        ],
                        "outputMappings": [
                            {
                                "definition": {"name": "result"},
                                "mappedOutput": {
                                    "definition": {"name": "result"},
                                    "solid": {"name": "div_2"},
                                },
                            }
                        ],
                        "solids": [{"name": "div_1"}, {"name": "div_2"}],
                    },
                    "inputs": [
                        {
                            "definition": {"name": "num"},
                            "dependsOn": [{"solid": {"name": "add_four"}}],
                        }
                    ],
                    "name": "div_four",
                    "outputs": [{"definition": {"name": "result"}, "dependedBy": []}],
                },
            },
            {
                "handleID": "div_four.div_1",
                "solid": {
                    "definition": {},
                    "inputs": [{"definition": {"name": "num"}, "dependsOn": []}],
                    "name": "div_1",
                    "outputs": [
                        {
                            "definition": {"name": "result"},
                            "dependedBy": [{"solid": {"name": "div_2"}}],
                        }
                    ],
                },
            },
            {
                "handleID": "div_four.div_2",
                "solid": {
                    "definition": {},
                    "inputs": [
                        {"definition": {"name": "num"}, "dependsOn": [{"solid": {"name": "div_1"}}]}
                    ],
                    "name": "div_2",
                    "outputs": [{"definition": {"name": "result"}, "dependedBy": []}],
                },
            },
        ],
    }
}

snapshots['TestComposites.test_composites[non_launchable_postgres_instance_lazy_repository] 1'] = {
    'pipelineOrError': {
        '__typename': 'Pipeline',
        'name': 'composites_pipeline',
        'solidHandles': [
            {
                'handleID': 'add_four',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'add_four',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'div_four'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1.adder_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1.adder_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2.adder_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2.adder_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'div_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'div_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'div_1'
                            },
                            {
                                'name': 'div_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'add_four'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'div_four',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four.div_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'div_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'div_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four.div_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'div_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'div_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            }
        ]
    }
}

snapshots['TestComposites.test_composites[non_launchable_postgres_instance_managed_grpc_env] 1'] = {
    'pipelineOrError': {
        '__typename': 'Pipeline',
        'name': 'composites_pipeline',
        'solidHandles': [
            {
                'handleID': 'add_four',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'add_four',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'div_four'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1.adder_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1.adder_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2.adder_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2.adder_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'div_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'div_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'div_1'
                            },
                            {
                                'name': 'div_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'add_four'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'div_four',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four.div_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'div_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'div_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four.div_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'div_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'div_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            }
        ]
    }
}

snapshots['TestComposites.test_composites[non_launchable_postgres_instance_multi_location] 1'] = {
    'pipelineOrError': {
        '__typename': 'Pipeline',
        'name': 'composites_pipeline',
        'solidHandles': [
            {
                'handleID': 'add_four',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'add_four',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'div_four'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1.adder_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1.adder_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2.adder_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2.adder_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'div_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'div_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'div_1'
                            },
                            {
                                'name': 'div_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'add_four'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'div_four',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four.div_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'div_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'div_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four.div_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'div_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'div_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            }
        ]
    }
}

snapshots['TestComposites.test_composites[non_launchable_sqlite_instance_deployed_grpc_env] 1'] = {
    'pipelineOrError': {
        '__typename': 'Pipeline',
        'name': 'composites_pipeline',
        'solidHandles': [
            {
                'handleID': 'add_four',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'add_four',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'div_four'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1.adder_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1.adder_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2.adder_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2.adder_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'div_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'div_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'div_1'
                            },
                            {
                                'name': 'div_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'add_four'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'div_four',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four.div_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'div_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'div_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four.div_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'div_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'div_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            }
        ]
    }
}

snapshots['TestComposites.test_composites[non_launchable_sqlite_instance_lazy_repository] 1'] = {
    'pipelineOrError': {
        '__typename': 'Pipeline',
        'name': 'composites_pipeline',
        'solidHandles': [
            {
                'handleID': 'add_four',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'add_four',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'div_four'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1.adder_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1.adder_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2.adder_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2.adder_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'div_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'div_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'div_1'
                            },
                            {
                                'name': 'div_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'add_four'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'div_four',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four.div_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'div_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'div_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four.div_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'div_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'div_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            }
        ]
    }
}

snapshots['TestComposites.test_composites[non_launchable_sqlite_instance_managed_grpc_env] 1'] = {
    'pipelineOrError': {
        '__typename': 'Pipeline',
        'name': 'composites_pipeline',
        'solidHandles': [
            {
                'handleID': 'add_four',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'add_four',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'div_four'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1.adder_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1.adder_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2.adder_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2.adder_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'div_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'div_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'div_1'
                            },
                            {
                                'name': 'div_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'add_four'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'div_four',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four.div_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'div_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'div_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four.div_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'div_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'div_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            }
        ]
    }
}

snapshots['TestComposites.test_composites[non_launchable_sqlite_instance_multi_location] 1'] = {
    'pipelineOrError': {
        '__typename': 'Pipeline',
        'name': 'composites_pipeline',
        'solidHandles': [
            {
                'handleID': 'add_four',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'add_four',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'div_four'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1.adder_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_1.adder_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'adder_1'
                            },
                            {
                                'name': 'adder_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2.adder_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'adder_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'adder_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'add_four.adder_2.adder_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'adder_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'adder_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four',
                'solid': {
                    'definition': {
                        'inputMappings': [
                            {
                                'definition': {
                                    'name': 'num'
                                },
                                'mappedInput': {
                                    'definition': {
                                        'name': 'num'
                                    },
                                    'solid': {
                                        'name': 'div_1'
                                    }
                                }
                            }
                        ],
                        'outputMappings': [
                            {
                                'definition': {
                                    'name': 'result'
                                },
                                'mappedOutput': {
                                    'definition': {
                                        'name': 'result'
                                    },
                                    'solid': {
                                        'name': 'div_2'
                                    }
                                }
                            }
                        ],
                        'solids': [
                            {
                                'name': 'div_1'
                            },
                            {
                                'name': 'div_2'
                            }
                        ]
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'add_four'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'div_four',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four.div_1',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                            ]
                        }
                    ],
                    'name': 'div_1',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                                {
                                    'solid': {
                                        'name': 'div_2'
                                    }
                                }
                            ]
                        }
                    ]
                }
            },
            {
                'handleID': 'div_four.div_2',
                'solid': {
                    'definition': {
                    },
                    'inputs': [
                        {
                            'definition': {
                                'name': 'num'
                            },
                            'dependsOn': [
                                {
                                    'solid': {
                                        'name': 'div_1'
                                    }
                                }
                            ]
                        }
                    ],
                    'name': 'div_2',
                    'outputs': [
                        {
                            'definition': {
                                'name': 'result'
                            },
                            'dependedBy': [
                            ]
                        }
                    ]
                }
            }
        ]
    }
}



import graphene
from dagster import check
from dagster.core.events import AssetLineageInfo, DagsterEventType
from dagster.core.execution.plan.objects import ErrorSource
from dagster.core.execution.stats import RunStepKeyStatsSnapshot

from ...implementation.events import construct_basic_params
from ...implementation.fetch_runs import get_run_by_id, get_step_stats
from ...implementation.loader import BatchRunLoader
from ..asset_key import GrapheneAssetKey, GrapheneAssetLineageInfo
from ..errors import GraphenePythonError, GrapheneRunNotFoundError
from ..metadata import GrapheneEventMetadataEntry
from ..runs import GrapheneStepEventStatus
from ..util import non_null_list
from .log_level import GrapheneLogLevel


class GrapheneMessageEvent(graphene.Interface):
    runId = graphene.NonNull(graphene.String)
    message = graphene.NonNull(graphene.String)
    timestamp = graphene.NonNull(graphene.String)
    level = graphene.NonNull(GrapheneLogLevel)
    stepKey = graphene.Field(graphene.String)
    solidHandleID = graphene.Field(graphene.String)
    eventType = graphene.Field(graphene.Enum.from_enum(DagsterEventType))

    class Meta:
        name = "MessageEvent"


class GrapheneDisplayableEvent(graphene.Interface):
    label = graphene.NonNull(graphene.String)
    description = graphene.String()
    metadataEntries = non_null_list(GrapheneEventMetadataEntry)

    class Meta:
        name = "DisplayableEvent"


class GrapheneMissingRunIdErrorEvent(graphene.ObjectType):
    invalidRunId = graphene.NonNull(graphene.String)

    class Meta:
        name = "MissingRunIdErrorEvent"


class GrapheneLogMessageEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent,)
        name = "LogMessageEvent"


class GrapheneRunEvent(graphene.Interface):
    pipelineName = graphene.NonNull(graphene.String)

    class Meta:
        name = "RunEvent"


class GrapheneRunEnqueuedEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneRunEvent)
        name = "RunEnqueuedEvent"


class GrapheneRunDequeuedEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneRunEvent)
        name = "RunDequeuedEvent"


class GrapheneRunStartingEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneRunEvent)
        name = "RunStartingEvent"


class GrapheneRunCancelingEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneRunEvent)
        name = "RunCancelingEvent"


class GrapheneRunCanceledEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneRunEvent)
        name = "RunCanceledEvent"


class GrapheneRunStartEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneRunEvent)
        name = "RunStartEvent"


class GrapheneRunSuccessEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneRunEvent)
        name = "RunSuccessEvent"


class GrapheneRunFailureEvent(graphene.ObjectType):
    error = graphene.Field(GraphenePythonError)

    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneRunEvent)
        name = "RunFailureEvent"


class GrapheneAlertStartEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneRunEvent)
        name = "AlertStartEvent"


class GrapheneAlertSuccessEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneRunEvent)
        name = "AlertSuccessEvent"


class GrapheneStepEvent(graphene.Interface):
    stepKey = graphene.Field(graphene.String)
    solidHandleID = graphene.Field(graphene.String)

    class Meta:
        name = "StepEvent"


class GrapheneExecutionStepStartEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneStepEvent)
        name = "ExecutionStepStartEvent"


class GrapheneExecutionStepRestartEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneStepEvent)
        name = "ExecutionStepRestartEvent"


class GrapheneExecutionStepUpForRetryEvent(graphene.ObjectType):
    error = graphene.Field(GraphenePythonError)
    secondsToWait = graphene.Field(graphene.Int)

    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneStepEvent)
        name = "ExecutionStepUpForRetryEvent"


class GrapheneExecutionStepSkippedEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneStepEvent)
        name = "ExecutionStepSkippedEvent"


class GrapheneObjectStoreOperationType(graphene.Enum):
    SET_OBJECT = "SET_OBJECT"
    GET_OBJECT = "GET_OBJECT"
    RM_OBJECT = "RM_OBJECT"
    CP_OBJECT = "CP_OBJECT"

    class Meta:
        name = "ObjectStoreOperationType"


class GrapheneObjectStoreOperationResult(graphene.ObjectType):
    op = graphene.NonNull(GrapheneObjectStoreOperationType)

    class Meta:
        interfaces = (GrapheneDisplayableEvent,)
        name = "ObjectStoreOperationResult"

    def resolve_metadataEntries(self, _graphene_info):
        from ...implementation.events import _to_metadata_entries

        return _to_metadata_entries(self.metadata_entries)  # pylint: disable=no-member


class GrapheneExpectationResult(graphene.ObjectType):
    success = graphene.NonNull(graphene.Boolean)

    class Meta:
        interfaces = (GrapheneDisplayableEvent,)
        name = "ExpectationResult"

    def resolve_metadataEntries(self, _graphene_info):
        from ...implementation.events import _to_metadata_entries

        return _to_metadata_entries(self.metadata_entries)  # pylint: disable=no-member


class GrapheneTypeCheck(graphene.ObjectType):
    success = graphene.NonNull(graphene.Boolean)

    class Meta:
        interfaces = (GrapheneDisplayableEvent,)
        name = "TypeCheck"

    def resolve_metadataEntries(self, _graphene_info):
        from ...implementation.events import _to_metadata_entries

        return _to_metadata_entries(self.metadata_entries)  # pylint: disable=no-member


class GrapheneFailureMetadata(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneDisplayableEvent,)
        name = "FailureMetadata"

    def resolve_metadataEntries(self, _graphene_info):
        from ...implementation.events import _to_metadata_entries

        return _to_metadata_entries(self.metadata_entries)  # pylint: disable=no-member


class GrapheneExecutionStepInputEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneStepEvent)
        name = "ExecutionStepInputEvent"

    input_name = graphene.NonNull(graphene.String)
    type_check = graphene.NonNull(GrapheneTypeCheck)


class GrapheneExecutionStepOutputEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneStepEvent, GrapheneDisplayableEvent)
        name = "ExecutionStepOutputEvent"

    output_name = graphene.NonNull(graphene.String)
    type_check = graphene.NonNull(GrapheneTypeCheck)


class GrapheneExecutionStepSuccessEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneStepEvent)
        name = "ExecutionStepSuccessEvent"


class GrapheneExecutionStepFailureEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneStepEvent)
        name = "ExecutionStepFailureEvent"

    error = graphene.Field(GraphenePythonError)
    errorSource = graphene.Field(graphene.Enum.from_enum(ErrorSource))
    failureMetadata = graphene.Field(GrapheneFailureMetadata)


class GrapheneHookCompletedEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneStepEvent)
        name = "HookCompletedEvent"


class GrapheneHookSkippedEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneStepEvent)
        name = "HookSkippedEvent"


class GrapheneHookErroredEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneStepEvent)
        name = "HookErroredEvent"

    error = graphene.Field(GraphenePythonError)


class GrapheneLogsCapturedEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent,)
        name = "LogsCapturedEvent"

    logKey = graphene.NonNull(graphene.String)
    stepKeys = graphene.List(graphene.NonNull(graphene.String))
    pid = graphene.Int()


def _construct_asset_event_metadata_params(event, metadata):
    metadata_params = {"label": metadata.label, "description": metadata.description}
    metadata_params.update(construct_basic_params(event))
    return metadata_params


class AssetEventMixin:
    assetKey = graphene.Field(GrapheneAssetKey)
    runOrError = graphene.NonNull("dagster_graphql.schema.pipelines.pipeline.GrapheneRunOrError")
    stepStats = graphene.NonNull(lambda: GrapheneRunStepStats)
    partition = graphene.Field(graphene.String)

    def __init__(self, event, metadata):
        self._event = event
        self._metadata = metadata

    def resolve_assetKey(self, _graphene_info):
        asset_key = self._metadata.asset_key

        if not asset_key:
            return None

        return GrapheneAssetKey(path=asset_key.path)

    def resolve_runOrError(self, graphene_info):
        return get_run_by_id(graphene_info, self._event.run_id)

    def resolve_stepStats(self, graphene_info):
        run_id = self.runId  # pylint: disable=no-member
        step_key = self.stepKey  # pylint: disable=no-member
        stats = get_step_stats(graphene_info, run_id, step_keys=[step_key])
        return stats[0]

    def resolve_metadataEntries(self, _graphene_info):
        from ...implementation.events import _to_metadata_entries

        return _to_metadata_entries(self._metadata.metadata_entries)

    def resolve_partition(self, _graphene_info):
        return self._metadata.partition


class GrapheneMaterializationEvent(graphene.ObjectType, AssetEventMixin):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneStepEvent, GrapheneDisplayableEvent)
        name = "MaterializationEvent"

    assetLineage = non_null_list(GrapheneAssetLineageInfo)

    def __init__(self, event, assetLineage=None, loader=None):
        self._asset_lineage = check.opt_list_param(assetLineage, "assetLineage", AssetLineageInfo)
        self._batch_run_loader = check.opt_inst_param(loader, "loader", BatchRunLoader)

        materialization = event.dagster_event.step_materialization_data.materialization
        super().__init__(**_construct_asset_event_metadata_params(event, materialization))
        AssetEventMixin.__init__(
            self,
            event=event,
            metadata=materialization,
        )

    def resolve_runOrError(self, graphene_info):
        from ..pipelines.pipeline import GrapheneRun

        if self._batch_run_loader:
            record = self._batch_run_loader.get_run_record_by_run_id(self._event.run_id)
            if not record:
                return GrapheneRunNotFoundError(self._event.run_id)

            return GrapheneRun(record)

        return super().resolve_runOrError(graphene_info)

    def resolve_assetLineage(self, _graphene_info):
        return [
            GrapheneAssetLineageInfo(
                assetKey=lineage_info.asset_key,
                partitions=lineage_info.partitions,
            )
            for lineage_info in self._asset_lineage
        ]


class GrapheneObservationEvent(graphene.ObjectType, AssetEventMixin):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneStepEvent, GrapheneDisplayableEvent)
        name = "ObservationEvent"

    def __init__(self, event):
        observation = event.dagster_event.asset_observation_data.asset_observation
        super().__init__(**_construct_asset_event_metadata_params(event, observation))
        AssetEventMixin.__init__(
            self,
            event=event,
            metadata=observation,
        )


class GrapheneHandledOutputEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneStepEvent, GrapheneDisplayableEvent)
        name = "HandledOutputEvent"

    output_name = graphene.NonNull(graphene.String)
    manager_key = graphene.NonNull(graphene.String)


class GrapheneLoadedInputEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneStepEvent)
        name = "LoadedInputEvent"

    input_name = graphene.NonNull(graphene.String)
    manager_key = graphene.NonNull(graphene.String)
    upstream_output_name = graphene.Field(graphene.String)
    upstream_step_key = graphene.Field(graphene.String)


class GrapheneObjectStoreOperationEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneStepEvent)
        name = "ObjectStoreOperationEvent"

    operation_result = graphene.NonNull(GrapheneObjectStoreOperationResult)


class GrapheneEngineEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneDisplayableEvent, GrapheneStepEvent)
        name = "EngineEvent"

    error = graphene.Field(GraphenePythonError)
    marker_start = graphene.Field(graphene.String)
    marker_end = graphene.Field(graphene.String)


class GrapheneStepExpectationResultEvent(graphene.ObjectType):
    class Meta:
        interfaces = (GrapheneMessageEvent, GrapheneStepEvent)
        name = "StepExpectationResultEvent"

    expectation_result = graphene.NonNull(GrapheneExpectationResult)


# Should be a union of all possible events
class GrapheneDagsterRunEvent(graphene.Union):
    class Meta:
        types = (
            GrapheneExecutionStepFailureEvent,
            GrapheneExecutionStepInputEvent,
            GrapheneExecutionStepOutputEvent,
            GrapheneExecutionStepSkippedEvent,
            GrapheneExecutionStepStartEvent,
            GrapheneExecutionStepSuccessEvent,
            GrapheneExecutionStepUpForRetryEvent,
            GrapheneExecutionStepRestartEvent,
            GrapheneLogMessageEvent,
            GrapheneRunFailureEvent,
            GrapheneRunStartEvent,
            GrapheneRunEnqueuedEvent,
            GrapheneRunDequeuedEvent,
            GrapheneRunStartingEvent,
            GrapheneRunCancelingEvent,
            GrapheneRunCanceledEvent,
            GrapheneRunSuccessEvent,
            GrapheneHandledOutputEvent,
            GrapheneLoadedInputEvent,
            GrapheneLogsCapturedEvent,
            GrapheneObjectStoreOperationEvent,
            GrapheneStepExpectationResultEvent,
            GrapheneMaterializationEvent,
            GrapheneObservationEvent,
            GrapheneEngineEvent,
            GrapheneHookCompletedEvent,
            GrapheneHookSkippedEvent,
            GrapheneHookErroredEvent,
            GrapheneAlertStartEvent,
            GrapheneAlertSuccessEvent,
        )
        name = "DagsterRunEvent"


class GraphenePipelineRunStepStats(graphene.Interface):
    runId = graphene.NonNull(graphene.String)
    stepKey = graphene.NonNull(graphene.String)
    status = graphene.Field(GrapheneStepEventStatus)
    startTime = graphene.Field(graphene.Float)
    endTime = graphene.Field(graphene.Float)
    materializations = non_null_list(GrapheneMaterializationEvent)
    expectationResults = non_null_list(GrapheneExpectationResult)

    class Meta:
        name = "PipelineRunStepStats"


class GrapheneRunMarker(graphene.ObjectType):
    startTime = graphene.Field(graphene.Float)
    endTime = graphene.Field(graphene.Float)

    class Meta:
        name = "RunMarker"


class GrapheneRunStepStats(graphene.ObjectType):
    runId = graphene.NonNull(graphene.String)
    stepKey = graphene.NonNull(graphene.String)
    status = graphene.Field(GrapheneStepEventStatus)
    startTime = graphene.Field(graphene.Float)
    endTime = graphene.Field(graphene.Float)
    materializations = non_null_list(GrapheneMaterializationEvent)
    expectationResults = non_null_list(GrapheneExpectationResult)
    attempts = non_null_list(GrapheneRunMarker)
    markers = non_null_list(GrapheneRunMarker)

    class Meta:
        interfaces = (GraphenePipelineRunStepStats,)
        name = "RunStepStats"

    def __init__(self, stats):
        self._stats = check.inst_param(stats, "stats", RunStepKeyStatsSnapshot)
        super().__init__(
            runId=stats.run_id,
            stepKey=stats.step_key,
            status=stats.status,
            startTime=stats.start_time,
            endTime=stats.end_time,
            materializations=stats.materialization_events,
            expectationResults=stats.expectation_results,
            attempts=[
                GrapheneRunMarker(startTime=attempt.start_time, endTime=attempt.end_time)
                for attempt in stats.attempts_list
            ],
            markers=[
                GrapheneRunMarker(startTime=marker.start_time, endTime=marker.end_time)
                for marker in stats.markers
            ],
        )



import sys
from unittest import mock

from dagster.utils.error import serializable_error_info_from_exc_info
from dagster_graphql.client.client_queries import (
    CLIENT_GET_REPO_LOCATIONS_NAMES_AND_PIPELINES_QUERY,
)
from dagster_graphql.implementation.utils import ErrorCapture
from dagster_graphql.schema.errors import GraphenePythonError
from dagster_graphql.test.utils import execute_dagster_graphql


def test_python_error():
    def func():
        raise Exception("bar")

    try:
        func()
    except:  # pylint: disable=W0702
        python_error = GraphenePythonError(serializable_error_info_from_exc_info(sys.exc_info()))

    assert python_error
    assert isinstance(python_error.message, str)
    assert isinstance(python_error.stack, list)
    assert len(python_error.stack) == 2
    assert "bar" in python_error.stack[1]


def test_error_capture(graphql_context):
    seen = []

    def _new_on_exc(exc_info):
        seen.append(exc_info)
        return ErrorCapture.default_on_exception(exc_info)

    ErrorCapture.on_exception = _new_on_exc

    with mock.patch(
        "dagster.core.workspace.context.BaseWorkspaceRequestContext.repository_locations",
        new_callable=mock.PropertyMock,
    ) as repo_locs_mock:
        repo_locs_mock.side_effect = Exception("oops all berries")
        execute_dagster_graphql(
            graphql_context,
            CLIENT_GET_REPO_LOCATIONS_NAMES_AND_PIPELINES_QUERY,
        )

    assert len(seen) == 1
